Class: Postsvg::Translation::Handlers::ClipPathHandler
- Inherits:
-
Object
- Object
- Postsvg::Translation::Handlers::ClipPathHandler
- Defined in:
- lib/postsvg/translation/handlers/clip_path_handler.rb
Class Method Summary collapse
-
.call(_element, _context) ⇒ Object
clipPath definitions are read at parse time into Svg::ClipPathRegistry.
Class Method Details
.call(_element, _context) ⇒ Object
clipPath definitions are read at parse time into Svg::ClipPathRegistry. Nothing to emit here.
9 |
# File 'lib/postsvg/translation/handlers/clip_path_handler.rb', line 9 def self.call(_element, _context); end |